extern crate core;

use std::ffi::CString;
use std::fmt::{Display, Formatter};

use xcb::x::{Keycode, Keysym, Timestamp};

pub mod chartbl;

#[derive(Copy, Clone, PartialEq, Eq)]
pub struct Position<T>(pub T, pub T);

pub enum Instruction<'a> {
    Delay(Timestamp),
    ButtonPress(u8),
    ButtonRelease(u8),
    MotionNotify(Position<i16>),
    KeyCodePress(Keycode),
    KeyCodeRelease(Keycode),
    KeySymPress(Keysym),
    KeySymRelease(Keysym),
    KeySym(Keysym),
    KeyStrPress(CString),
    KeyStrRelease(CString),
    KeyStr(CString),
    String(&'a str),
    ExecBlock(&'a str),
    ExecNoBlock(&'a str),
}

impl Display for Instruction<'_> {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match self {
                Instruction::Delay(d) => format!("Delay {}", d),
                Instruction::ButtonPress(b) => format!("ButtonPress {}", b),
                Instruction::ButtonRelease(b) => format!("ButtonRelease {}", b),
                Instruction::MotionNotify(pos) => format!("MotionNotify {} {}", pos.0, pos.1),
                Instruction::KeyCodePress(kc) => format!("KeyCodePress {}", kc),
                Instruction::KeyCodeRelease(kc) => format!("KeyCodeRelease {}", kc),
                Instruction::KeySymPress(ks) => format!("KeySymPress {}", ks),
                Instruction::KeySymRelease(ks) => format!("KeySymRelease {}", ks),
                Instruction::KeySym(ks) => format!("KeySym {}", ks),
                Instruction::KeyStrPress(kstr) =>
                    format!("KeyStrPress {}", kstr.to_str().unwrap()),
                Instruction::KeyStrRelease(kstr) =>
                    format!("KeyStrRelease {}", kstr.to_str().unwrap()),
                Instruction::KeyStr(kstr) => format!("KeyStr {}", kstr.to_str().unwrap()),
                Instruction::String(str) => format!("String {}", str),
                Instruction::ExecBlock(cmd) => format!("ExecBlock {}", cmd),
                Instruction::ExecNoBlock(cmd) => format!("ExecNoBlock {}", cmd),
            },
        )
    }
}
